﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ColorPicker
{
	public partial class MainForm : Form
	{
		private readonly string POINT_TEMPLATE = "X:{0}" + Environment.NewLine + "Y: {1}";
		private readonly string RGB_TEMPLATE = "R: {0}" + Environment.NewLine + "G: {1}" + Environment.NewLine + "B: {2}";
		private readonly string HEX_TEMPLATE = "HEX: #{0}";

		private string clip;

		private Thread thread;
		private bool pickFlag = true;

		public MainForm()
		{
			InitializeComponent();

			InitInstance();
		}

		private void InitInstance()
		{
			thread = new Thread(() => DoWork());
			thread.Start();

			this.FormClosing += MainForm_FormClosing;

			this.TopMost = true;
		}

		

		private void DoWork()
		{
			while (true)
			{
				if (!pickFlag)
					continue;

				int x = Control.MousePosition.X;
				int y = Control.MousePosition.Y;
				Color color = PickColor(x, y);

				PrintResult(x, y, color);
			}
		}

		private Color PickColor(int x, int y)
		{
			Size size = new Size(1, 1);
			Bitmap bitmap = new Bitmap(1, 1);
			Graphics g = Graphics.FromImage(bitmap);
			g.CopyFromScreen(x, y, 0, 0, size);

			return bitmap.GetPixel(0, 0);
		}

		private void PrintResult(int x, int y, Color color)
		{
			if (this.InvokeRequired)
			{
				Invoke((MethodInvoker)delegate () {
					PrintResult(x, y, color);
				});
			}
			else
			{
				pointLabel.Text = string.Format(POINT_TEMPLATE, x, y);

				colorPanel.BackColor = color;

				int r = color.R;
				int g = color.G;
				int b = color.B;
				string hex = color.R.ToString("X2") + color.G.ToString("X2") + color.B.ToString("X2");
				rgbLabel.Text = string.Format(RGB_TEMPLATE, r, g, b);
				hexLabel.Text = string.Format(HEX_TEMPLATE, hex);

				clip = $"RGB: {r}, {g}, {b} HEX: {hex}";
			}
		}


		private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			thread.Abort();
		}

		private void MainForm_KeyPress(object sender, KeyPressEventArgs e)
		{
			if (e.KeyChar == (char)Keys.Space)
			{
				pickFlag = !pickFlag;

				//toolTip.ToolTipTitle = "Info";
				//toolTip.IsBalloon = true;
				////toolTip.SetToolTip(helpLabel, pickFlag ? "Resumed" : "Paused");
				//toolTip.Show(pickFlag ? "Resumed" : "Paused", helpLabel);
			}
		}

		private void MainForm_KeyDown(object sender, KeyEventArgs e)
		{
			if (e.Control && e.KeyCode == Keys.C)
			{
				Clipboard.SetText(clip);

				//toolTip.ToolTipTitle = "Info";
				//toolTip.IsBalloon = true;
				//toolTip.SetToolTip(helpLabel, "Copied");
			}
		}
	}
}
